/*--------------------------------------------------------------------------*/
/*----------------------------- File cyn2jlf.C -----------------------------*/
/*--------------------------------------------------------------------------*/
/*--                							  --*/
/*--  Little translator: translate MMCF problems in the format used by    --*/
/*--  C. Barnerth and P. Vance, that allows fractional capacities, to     --*/
/*--  the JL Format and (evenctually) integer capacities: this is done    --*/
/*--  by letting the user to specify a factor for which all supplies,     --*/
/*--  demands and capacities will be multiplied - however, it's user's    --*/
/*--  responsibility that the final capacities will then be integer.      --*/
/*--                                                                      --*/
/*--  The translator produces both the formats of demand/supply file: a   --*/
/*--  .sup file where commodities are aggregated by origin, and a .od     --*/
/*--  file where each O/D pair is considered a commodity.                 --*/
/*--                                                                      --*/
/*--  The assumptions on the input file are that (obviously) the MMCFs    --*/
/*--  have SPT-subproblems, and that all the O/D pairs having the same    --*/
/*--  origin *and* product are listed consecutively in the file.          --*/
/*--                                                                      --*/
/*--                            VERSION 0.90 beta                         --*/
/*--                	       23 - 07 - 1996			       	  --*/
/*--                     			                          --*/
/*-- 		     Original Idea and Implementation by:		  --*/
/*--                							  --*/
/*--			       Antonio Frangioni       			  --*/
/*--                							  --*/
/*--   			   Operations Research Group			  --*/
/*--			  Dipartimento di Informatica			  --*/
/*--   			     Universita' di Pisa			  --*/
/*--                							  --*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

#include <fstream.h>
#include <strstream.h>
#include <stdlib.h>
#include <string.h>

/*--------------------------------------------------------------------------*/

inline double str2double( char *str )
{
 double i;

 istrstream( str , sizeof( str ) ) >> i;

 return( i );
 }

/*--------------------------------------------------------------------------*/

main( int argc , char **argv )
{
 if( argc < 2 )
 {
  cerr << "Usage: cyn2jlf input_file_name [factor]" << endl;
  exit( -1 );
  }

 double fctr = 1;

 if( argc == 3 )
  fctr = str2double( argv[ 2 ] );

 ifstream iFile;
 iFile.open( argv[ 1 ] , ios::in );

 if( ! iFile )
 {
  cerr << "ERROR: opening file " << argv[ 1 ] << endl;
  exit( 1 );
  }

 int l = strlen( argv[ 1 ] );
 char *Name = new char[ l + 5 ];
 strcpy( Name , argv[ 1 ] );

 strcpy( Name + l , ".arc" );

 ofstream oFile1;
 oFile1.open( Name , ios::out );

 strcpy( Name + l , ".mut" );

 ofstream oFile2;
 oFile2.open( Name , ios::out );

 int NumArchi;
 int NumNodi;

 iFile >> NumNodi;
 iFile >> NumArchi;

 for( int i = 1 ; i <= NumArchi ; )
 {
  int from;
  int to;
  int prod;
  double cost;
  double cap;

  iFile >> from;
  iFile >> to;
  iFile >> prod;
  iFile >> cost;
  iFile >> cap;

  oFile1 << from << "\t" << to << "\t" << prod << "\t" << cost
         << "\t-1\t-1\t-1\t" << i << endl;

  oFile2 << i++ << "\t" << cap * fctr << endl;
  }

 oFile1.close();
 oFile2.close();

 strcpy( Name + l , ".od" );
 oFile1.open( Name , ios::out );

 strcpy( Name + l , ".sup" );
 oFile2.open( Name , ios::out );

 int NumComm = 0;
 double totflw = 0;
 int currprd;
 int currorg;

 // the assumption over the file format is that all the O/D pairs
 // relative to a single product that have also the same O are
 // listed consecutively in the file

 iFile >> i;

 for( ; i-- ; )
 {
  int prod;
  int from;
  int to;
  double flow;

  iFile >> prod;
  iFile >> from;
  iFile >> to;
  iFile >> flow;

  flow *= fctr;

  oFile1 << from << "\t" << to << "\t" << prod << "\t" << flow << endl;

  if( ! NumComm )
  {
   currprd = prod;
   currorg = from;
   }

  if( ( prod != currprd ) || ( from != currorg ) )
  {
   oFile2 << currorg << "\t-1\t" << currprd << "\t" << totflw << endl;

   totflw = 0;
   currprd = prod;
   currorg = from;
   }

  oFile2 << from << "\t" << to << "\t" << prod << "\t" << flow << endl;

  totflw += flow;

  if( prod > NumComm )
   NumComm = prod;
  }

 oFile2 << currorg << "\t-1\t" << currprd << "\t" << totflw << endl;

 oFile1.close();
 oFile2.close();

 strcpy( Name + l , ".nod" );
 oFile2.open( Name , ios::out );

 oFile2 << NumComm << "\t" << NumNodi << "\t" << NumArchi << "\t" << NumArchi
        << endl;

 oFile2.close();

 }  // end( main )

/*--------------------------------------------------------------------------*/
/*-------------------------- End File cyn2jlf.C ----------------------------*/
/*--------------------------------------------------------------------------*/
